severity = brutal
verbose = 8
exclude = RequireEmacsFileVariables RequireRcsKeywords Tics::ProhibitLongLines Documentation::RequirePODUseEncodingUTF8 Modules::RequireExplicitPackage Modules::RequireExplicitInclusion

[Lax::RequireExplicitPackage::ExceptForPragmata]
allowed_pragmata = perlversion strict warnings utf8

[CodeLayout::RequireTidyCode]
perltidyrc = xt/author/perltidy.rc

[Documentation::PodSpelling]
stop_words = WSDL plugin typemap uri CGI Perldoc perldoc annonations PASSed

[Documentation::RequirePodSections]
lib_sections = NAME | VERSION | SYNOPSIS | DESCRIPTION | SUPPORT | AUTHOR | COPYRIGHT AND LICENSE

[InputOutput::RequireCheckedSyscalls]
functions = :builtins
exclude_functions = print

[Subroutines::ProhibitCallsToUndeclaredSubs]
exempt_subs = Moose::has Moose::with MooseX::Has::Sugar::coerce MooseX::Has::Sugar::lazy MooseX::Has::Sugar::lazy_build MooseX::Has::Sugar::required MooseX::Has::Sugar::ro Path::Class::file Try::Tiny::catch Try::Tiny::try
